% Input :
%		 featureConstructors : feature constructors returned by getKKFeatureConstructors code
%		 origFeatures : original features for the dataset

% Output :
%		 featureVectors : a matrix of dimensionality numOrigFeatures x newFeatureDim.
%						  Note that the features are returned in a transposed format that can be directly passed on to Liblinear or Libsvm
%		 timeTaken : cpu time taken to perform this task

function [featureVectors timeTaken] = applyKKFeatureConstructors(featureConstructors,origFeatures)
	% Some features are simply constants
	idxNonTrivialFeatures = featureConstructors.N > 0;
	idxTrivialFeatures = featureConstructors.N == 0;
	numNonTrivialFeatures = sum(idxNonTrivialFeatures);
	numTrivialFeatures = sum(idxTrivialFeatures);
	
	coefficientOfConstantFeatures = min(featureConstructors.a(featureConstructors.N == 0));
	
	input1 = int8(featureConstructors.W);
	input3 = int32(numNonTrivialFeatures);
	input4 = int32(featureConstructors.N(featureConstructors.N > 0));
	
	startTime = cputime;
	featureVectors = doKKFeatureConstructionStep(input1,origFeatures,input3,input4);
	timeTaken = cputime - startTime;
	
	temp = featureConstructors.a(featureConstructors.N > 0)';
	tempFeatureVectors = bsxfun(@times,temp,featureVectors);
	
	if numTrivialFeatures > 0
		featureVectors = coefficientOfConstantFeatures*ones(size(tempFeatureVectors,1),featureConstructors.D);
		featureVectors(:,idxNonTrivialFeatures) = tempFeatureVectors;
	else
		featureVectors = tempFeatureVectors;
	end
end