clear;

dirName = 'KK Exponential Expt v1';
mkdir(dirName);

% Load the parameters of the experiment
load params.mat

% Experiment only on spambase to begin with
params.datasets = {'nursery'};
params.numDatasets = size(params.datasets,2);
params.datasetDimensions = [8];

KKExpAvgRecall = zeros(params.numProjDimen,params.nRuns,params.numDatasets);
KKExpTrainingTime = zeros(params.numProjDimen,params.nRuns,params.numDatasets);
KKExpTestTime = zeros(params.numProjDimen,params.nRuns,params.numDatasets);

version = 1;

COpt = 100;

NMax = 15;

coefficientForFirstTaylorTerm = 1;
coefficientForSecondTaylorTerm = 1;
for datasetCounter = 1:params.numDatasets
    fprintf([params.datasets{datasetCounter} '\n']);
	
	for runCounter = 1 : params.nRuns
        % Load the dataset
        [trainingFeatures trainingLabels testFeatures testLabels sigma nClasses] = loadNormalizedSplit(params.datasets{datasetCounter},runCounter,params);
		
		if strcmp(params.datasets{datasetCounter},'mnist')
			% Throw away dimensions with no variance
			stddev = std(trainingFeatures,0,2);
			trainingFeatures = trainingFeatures(stddev > 0.5, :);
			testFeatures = testFeatures(stddev > 0.5, :);
		end
		
		% Make labels suitable to be passed on to libsvm
		trainingLabels(trainingLabels ~= 1) = -1;
		testLabels(testLabels ~= 1) = -1;
		
		numTrainingFeatures = length(trainingLabels);
		numTestFeatures = length(testLabels);
		
		% What is the longest vector in the training set ?
		normalizeLength = max(sqrt(sum(trainingFeatures.^2)));
		
		% Make all vector short !
		trainingFeatures = trainingFeatures/normalizeLength;
		testFeatures = testFeatures/normalizeLength;
		
		numFeaturesToEstimateSigma = min(numTrainingFeatures,1000);
		
		% Recalculate sigma with the shortened vectors
		sigma = sum(sum(dist(trainingFeatures(:,1:numFeaturesToEstimateSigma)).^2))/(numFeaturesToEstimateSigma*(numFeaturesToEstimateSigma-1));
		
		% Get the kernel matrix generator function
		kernelFunction = getKernelMatrixGenerator('exp');
		
		trainingFeatureConstructionTimeForLargest = 0;
		
		for dimCounter = params.numProjDimen : -1 : 1
			% Get the Taylor feature generators for the largest number of features and save them for posterity
			if dimCounter == params.numProjDimen
				featureConstructors = getKKFeatureConstructors('exp',sigma,params.projDimen(dimCounter),size(trainingFeatures,1),NMax,version);
				
				disp('Starting Training feature construction!');
				
				[taylorTrainingFeatures trainingFeatureConstructionTimeForLargest] = applyKKFeatureConstructors(featureConstructors,trainingFeatures);
				
				disp('Training feature construction done!');
				
				% Taylor Feature means and standard deviation
				taylorFeaturesMean = mean(taylorTrainingFeatures);
				taylorTrainingFeatures = bsxfun(@minus,taylorFeaturesMean,taylorTrainingFeatures);
				taylorFeaturesStd = std(taylorTrainingFeatures);
				taylorTrainingFeatures = bsxfun(@rdivide,taylorTrainingFeatures,taylorFeaturesStd);

				% Longest Taylor feature
				taylorNormalizeLength = max(sqrt(sum(taylorTrainingFeatures.^2,2)));
				taylorTrainingFeatures = taylorTrainingFeatures/taylorNormalizeLength;
				
				% What is the ratio of time taken construction of the first k features to that taken to construct all the features
				featureComplexityRatios = cumsum(featureConstructors.N);
				featureComplexityRatios = featureComplexityRatios/featureComplexityRatios(end);
				
				% Save the feature constructor for future use
				save(sprintf('%s/KKExp_featureConstructor_%s_split%d.mat',dirName,params.datasets{datasetCounter},runCounter),'featureConstructors');
				
			% Else for smaller number of features simply take the first few features. Note : the C code returns transposed features !
			else
				taylorTrainingFeatures = taylorTrainingFeatures(:,1:params.projDimen(dimCounter));
			end
			
			% Since this is version 1 the original features need to go in - the constant term can be left out since the SVM b will take care of it
			sparseTaylorTrainingFeatures = sparse([trainingFeatures' taylorTrainingFeatures]);
			
			disp('Sparse training feature construction done!');
			
			% Start Training !

			startTime = cputime;
			model = lin_train(trainingLabels,sparseTaylorTrainingFeatures,sprintf('-c %f, -s 2 -B 1 -q',COpt));
			trainingTime = cputime - startTime;
			
			disp('Training done!');
			
			% Training time includes liblinear time + feature construction time.
			% Since features are not constructed afresh, simply use a scaled version of the time taken to construct the features of largest dimensionality
			KKExpTrainingTime(dimCounter, runCounter, datasetCounter) = trainingTime + trainingFeatureConstructionTimeForLargest*featureComplexityRatios(params.projDimen(dimCounter));
			
			save(sprintf('%s/KKExp_model_%s_split%d_%dFeatures.mat',dirName,params.datasets{datasetCounter},runCounter,params.projDimen(dimCounter)),'model');
			save([dirName '/KKExp_TrainingTime.mat'], 'KKExpTrainingTime');
		end
		
		% Start testing vectors in batches
		numBatches = ceil(numTestFeatures/params.batchSize);
		numCorrectlyClassified = zeros(params.numProjDimen,1);
		testTime = zeros(params.numProjDimen,1);
		
		% Load the appropriate feature constructor
		featureConstructors = load(sprintf('%s/KKExp_featureConstructor_%s_split%d.mat',dirName,params.datasets{datasetCounter},runCounter));
		featureConstructors = featureConstructors.featureConstructors;
		
		% What is the ratio of time taken construction of the first k features to that taken to construct all the features
		featureComplexityRatios = cumsum(featureConstructors.N);
		featureComplexityRatios = featureComplexityRatios/featureComplexityRatios(end);
		
        disp('Testing started');
		
		for batchCounter = 1 : numBatches
            disp(sprintf('batchCounter = %d',batchCounter));
			
			% Which all vectors fall in this batch ?
			idxForThisBatch = (batchCounter-1)*params.batchSize+1:min(batchCounter*params.batchSize,numTestFeatures);
			testFeaturesForThisBatch = testFeatures(:,idxForThisBatch);
			
			for dimCounter = params.numProjDimen : -1 : 1
				% Generate taylor features for the largest feature dimensionality
				if dimCounter == params.numProjDimen
					[taylorTestFeaturesForThisBatch testFeatureConstructionTimeForLargest] = applyKKFeatureConstructors(featureConstructors,testFeaturesForThisBatch);
					testTime(dimCounter) = testTime(dimCounter) + testFeatureConstructionTimeForLargest*featureComplexityRatios(params.projDimen(dimCounter));
					taylorTestFeaturesForThisBatch = bsxfun(@minus,taylorFeaturesMean,taylorTestFeaturesForThisBatch);
					taylorTestFeaturesForThisBatch = bsxfun(@rdivide,taylorTestFeaturesForThisBatch,taylorFeaturesStd);
					taylorTestFeaturesForThisBatch = taylorTestFeaturesForThisBatch/taylorNormalizeLength;
				% Else for smaller number of features simply take the first few features and estimate the time it should have taken to calculate them
				else
					taylorTestFeaturesForThisBatch = taylorTestFeaturesForThisBatch(:,1:params.projDimen(dimCounter));
					testTime(dimCounter) = testTime(dimCounter) + testFeatureConstructionTimeForLargest*featureComplexityRatios(params.projDimen(dimCounter));
				end
				
				% Load the appropriate model and extract the relevant parameters
				model = load(sprintf('%s/KKExp_model_%s_split%d_%dFeatures.mat',dirName,params.datasets{datasetCounter},runCounter,params.projDimen(dimCounter)));
				model = model.model;
				w = model.w(1:length(model.w)-1);
				b = model.w(end);
				
				% Since this is version 1, the original features also have to go in
				extendedTaylorTestFeaturesForThisBatch = [testFeaturesForThisBatch' taylorTestFeaturesForThisBatch];
				
				% Classify using the model obtained above and note the time taken
				startTime = cputime;
				decisionValues = extendedTaylorTestFeaturesForThisBatch*w' + b;
				testTime(dimCounter) = testTime(dimCounter) + (cputime - startTime);
				
				% How many did we get correctly ?
				decisionValues = decisionValues * model.Label(1);
				numCorrectlyClassified(dimCounter) = numCorrectlyClassified(dimCounter) + sum(testLabels(idxForThisBatch) == sign(decisionValues));
			end
		end
		
		KKExpAvgRecall(:, runCounter, datasetCounter) = numCorrectlyClassified/numTestFeatures;
		KKExpTestTime(:, runCounter, datasetCounter) = testTime;
		
		save([dirName '/KKExp_AvgRecall.mat'], 'KKExpAvgRecall');
		save([dirName '/KKExp_TestTime.mat'], 'KKExpTestTime');
		
		clear temp1 temp2 temp3 temp4 temp5
	end
end

save([dirName '/KKExp_AvgRecall.mat'], 'KKExpAvgRecall');
save([dirName '/KKExp_TrainingTime.mat'], 'KKExpTrainingTime');
save([dirName '/KKExp_TestTime.mat'], 'KKExpTestTime');
