% Test the accuracy of the proposed method on randomly generated point sets

% Various values of input dimension
dValues = [10 20 50 100 200];
numdValues = length(dValues);

% Various values of embedding dimension
DValues = [10,20,50,100,200,500,1000,2000,5000];
numDValues = length(DValues);

% Various kernels
kernels = {'exp'};
%kernels = {'homopoly'};
%kernels = {'poly'};
numKernels = length(kernels);

% ... and their parameters
sigma = 0;		% To be decided later
p = 10;

% Maximum complexity of a feature
NMax = 50;
numRuns = 5;
numPoints = 100;
normalize = 2;

% Put version = 0 to not use H0/1 (see paper section 6.1 of the paper) and put version = 1 to use H0/1
version = 1;

accuracies = zeros(numdValues,numDValues,numKernels);

for dCounter = 1:numdValues
	% Output some things to watch on the screen !!
	fprintf('************************************************\n');
	fprintf('d = %d\n',dValues(dCounter));

	d = dValues(dCounter);
	
	for runCounter = 1:numRuns
		fprintf('run = %d\n',runCounter);
		% The input vectors
		X = randn(d,numPoints);

		% Normalize the dataset
		if normalize == 0
			radius = 1;
		elseif normalize == 1
			radius = max(sum(abs(X),1));
		elseif normalize == 2
			radius = sqrt(max(sum(X.^2)));
		end
		
		Xnorm = X/radius;

		% Calculate the scale of the dataset
		sigma = sum(sum(dist(X).^2))/(numPoints*(numPoints-1));

		for kernelCounter = 1:numKernels
		
			% Get the matrix generator for this kernel
			kernel = getKernelMatrixGenerator(kernels{kernelCounter});
			
			% Set the coefficients of the taylor terms that are not getting estimated
			coefficientForFirstTaylorTerm = 1;
			
			% Set the kernel paramters and get the true kernel values
			switch(kernels{kernelCounter}),
				case 'exp',
					fprintf('Calculating Kernel for exp\n');
					kArgs = sigma;
					K = kernel(X,X,kArgs);
					coefficientForSecondTaylorTerm = 1;
				case 'poly',
					kArgs = p;
					K = kernel(Xnorm,Xnorm,kArgs);
					coefficientForSecondTaylorTerm = p;
				case 'homopoly',
					kArgs = p;
					K = kernel(Xnorm,Xnorm,kArgs);
					coefficientForSecondTaylorTerm = 1;
				otherwise,
					fprintf('Unknown Kernel type\n');
					keyboard;
			end

			% Only need to compare inter point kernel values
			K = K - diag(diag(K));

			% Get the feature generators
			featureConstructors = getKKFeatureConstructors(kernels{kernelCounter},kArgs,DValues(numDValues),d,NMax,version);
			
			% Apply them to the features
			if strcmp(kernels{kernelCounter},'exp')
				fprintf('Applying features to exp\n');
				[embeddedFeatures timeTaken] = applyKKFeatureConstructors(featureConstructors,X);
			else
				[embeddedFeatures timeTaken] = applyKKFeatureConstructors(featureConstructors,Xnorm);
			end

			for DCounter = 1: numDValues

				D = DValues(DCounter);
				embeddedFeaturesForThis = embeddedFeatures(:,1:D)/sqrt(D);

				% The estimated kernel values
				K1 = embeddedFeaturesForThis*embeddedFeaturesForThis';
				
				% Add the contribution of non estimated terms
				if ~strcmp(kernels{kernelCounter},'homopoly') & version == 1
					if strcmp(kernels{kernelCounter},'exp')
						K1 = K1/4 + coefficientForFirstTaylorTerm + coefficientForSecondTaylorTerm*(X'*X)/sigma;
					else
						K1 = K1/4 + coefficientForFirstTaylorTerm + coefficientForSecondTaylorTerm*(Xnorm'*Xnorm);
					end
				end
				
				% Only need to compare inter point values
				K1 = K1 - diag(diag(K1));

				acc = sum(sum(abs(K1 - K)))/(numPoints^2 - numPoints);
				
				accuracies(dCounter,DCounter,kernelCounter) = accuracies(dCounter,DCounter,kernelCounter) + acc;

				clear X1 K1
				save accuracies.txt accuracies '-ascii';
			end
			clear K
		end
		clear X
	end
end

accuracies = accuracies / numRuns;
save accuracies.txt accuracies '-ascii';