% Input :
%       datasetName : name of the dataset to load
%       splitNumber : which split to load
%       params : kernel learning parameters
% 
% Output :
%       normalized feature vectors for training and testing along with true labels

function [trainingFeatures trainingLabels testFeatures testLabels sigma nClasses] = loadNormalizedSplit(datasetName,splitNumber,params)
        
    % Load the dataset
    features = load([datasetName '.mtx'],'-mat');
	features = features.mtx';
    labels = load([datasetName '_binary.truth'],'-mat');
	labels = labels.lbl;
	labels = labels - min(labels) + 1;
    
	nData = length(labels);
    nClasses = max(labels);
    
    % Load the Split
    load(sprintf('%s_split%d.spl',datasetName,splitNumber),'-mat');
	features = bsxfun(@minus,features,split.avg);;
	features = bsxfun(@rdivide,features',split.stddev')';
    
    idxTraining = min(round(params.trainingFraction*nData),params.maxTraining);
	
	trainInds = split.idx(1:idxTraining);
	testInds = split.idx(idxTraining+1:nData);
	
    % Create the training and test sets
    trainingFeatures = features(:,trainInds);
    trainingLabels = labels(trainInds);
    testFeatures = features(:,testInds);
    testLabels = labels(testInds);
    
    % Send back the value of sigma
    sigma = split.sigma;
end 
